@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2023 Derick Rethans                               |
+   | Copyright (c) 2002-2024 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -206,7 +206,7 @@ int xdebug_lib_set_mode(const char *mode)
 	return result;
 }
 
-#if __linux__
+#if HAVE_XDEBUG_CONTROL_SOCKET_SUPPORT
 int xdebug_lib_set_control_socket_granularity(char *value)
 {
 	if (strcmp(value, "no") == 0 || value[0] == '\0') {
@@ -214,8 +214,17 @@ int xdebug_lib_set_control_socket_granularity(char *value)
 		return 1;
 	}
 
-	XINI_BASE(control_socket_granularity) = XDEBUG_CONTROL_SOCKET_TIME;
-	XINI_BASE(control_socket_threshold_ms) = 25;
+	if (strcmp(value, "default") == 0) {
+		XINI_BASE(control_socket_granularity) = XDEBUG_CONTROL_SOCKET_DEFAULT;
+		XINI_BASE(control_socket_threshold_ms) = 25;
+		return 1;
+	}
+
+	if (strcmp(value, "time") == 0) {
+		XINI_BASE(control_socket_granularity) = XDEBUG_CONTROL_SOCKET_TIME;
+		XINI_BASE(control_socket_threshold_ms) = 25;
+		return 1;
+	}
 
 	return 0;
 }
@@ -325,26 +334,61 @@ const char *xdebug_lib_mode_from_value(int mode)
 	}
 }
 
-static const char *find_in_globals(const char *element)
+const char *xdebug_lib_find_in_globals(const char *element, const char **found_in_global)
 {
 	zval *trigger_val = NULL;
 	const char *env_value = getenv(element);
+	zval *st;
 
+	/* Elements in Superglobal Symbols */
+	st = zend_hash_str_find_deref(&EG(symbol_table), "_GET", strlen("_GET"));
+	if (st && (trigger_val = zend_hash_str_find_deref(Z_ARRVAL_P(st), element, strlen(element))) != NULL) {
+		*found_in_global = "GET";
+		return Z_STRVAL_P(trigger_val);
+	}
+
+	st = zend_hash_str_find_deref(&EG(symbol_table), "_POST", strlen("_POST"));
+	if (st && (trigger_val = zend_hash_str_find_deref(Z_ARRVAL_P(st), element, strlen(element))) != NULL) {
+		*found_in_global = "POST";
+		return Z_STRVAL_P(trigger_val);
+	}
+
+	st = zend_hash_str_find_deref(&EG(symbol_table), "_COOKIE", strlen("_COOKIE"));
+	if (st && (trigger_val = zend_hash_str_find_deref(Z_ARRVAL_P(st), element, strlen(element))) != NULL) {
+		*found_in_global = "COOKIE";
+		return Z_STRVAL_P(trigger_val);
+	}
+
+	/* Actual Superglobals */
+	if ((trigger_val = zend_hash_str_find_deref(Z_ARR(PG(http_globals)[TRACK_VARS_GET]), element, strlen(element))) != NULL) {
+		*found_in_global = "GET";
+		return Z_STRVAL_P(trigger_val);
+	}
+
+	if ((trigger_val = zend_hash_str_find_deref(Z_ARR(PG(http_globals)[TRACK_VARS_POST]), element, strlen(element))) != NULL) {
+		*found_in_global = "POST";
+		return Z_STRVAL_P(trigger_val);
+	}
+
+	if ((trigger_val = zend_hash_str_find_deref(Z_ARR(PG(http_globals)[TRACK_VARS_COOKIE]), element, strlen(element))) != NULL) {
+		*found_in_global = "COOKIE";
+		return Z_STRVAL_P(trigger_val);
+	}
+
+	/* Environment. This goes last. */
 	if (env_value) {
+		*found_in_global = "ENV";
 		return env_value;
 	}
 
-	if (
-		(
-			(trigger_val = zend_hash_str_find(Z_ARR(PG(http_globals)[TRACK_VARS_ENV]), element, strlen(element))) != NULL
-		) || (
-			(trigger_val = zend_hash_str_find(Z_ARR(PG(http_globals)[TRACK_VARS_GET]), element, strlen(element))) != NULL
-		) || (
-			(trigger_val = zend_hash_str_find(Z_ARR(PG(http_globals)[TRACK_VARS_POST]), element, strlen(element))) != NULL
-		) || (
-			(trigger_val = zend_hash_str_find(Z_ARR(PG(http_globals)[TRACK_VARS_COOKIE]), element, strlen(element))) != NULL
-		)
-	) {
+	st = zend_hash_str_find_deref(&EG(symbol_table), "_ENV", strlen("_ENV"));
+	if (st && (trigger_val = zend_hash_str_find_deref(Z_ARRVAL_P(st), element, strlen(element))) != NULL) {
+		*found_in_global = "ENV";
+		return Z_STRVAL_P(trigger_val);
+	}
+
+	if ((trigger_val = zend_hash_str_find_deref(Z_ARR(PG(http_globals)[TRACK_VARS_ENV]), element, strlen(element))) != NULL) {
+		*found_in_global = "ENV";
 		return Z_STRVAL_P(trigger_val);
 	}
 
@@ -431,11 +475,12 @@ static int trigger_enabled(int for_mode, char **found_trigger_value)
 {
 	const char *trigger_value = NULL;
 	const char *trigger_name = "XDEBUG_TRIGGER";
+	const char *found_in_global;
 
 	xdebug_log(XLOG_CHAN_CONFIG, XLOG_DEBUG, "Checking if trigger 'XDEBUG_TRIGGER' is enabled for mode '%s'", xdebug_lib_mode_from_value(for_mode));
 
 	/* First we check for the generic 'XDEBUG_TRIGGER' option */
-	trigger_value = find_in_globals(trigger_name);
+	trigger_value = xdebug_lib_find_in_globals(trigger_name, &found_in_global);
 
 	/* If not found, we fall back to the per-mode name for backwards compatibility reasons */
 	if (!trigger_value) {
@@ -449,7 +494,7 @@ static int trigger_enabled(int for_mode, char **found_trigger_value)
 
 		if (trigger_name) {
 			xdebug_log(XLOG_CHAN_CONFIG, XLOG_INFO, "Trigger value for 'XDEBUG_TRIGGER' not found, falling back to '%s'", trigger_name);
-			trigger_value = find_in_globals(trigger_name);
+			trigger_value = xdebug_lib_find_in_globals(trigger_name, &found_in_global);
 		}
 	}
 
@@ -758,6 +803,31 @@ zend_string *xdebug_wrap_location_around_function_name(const char *prefix, zend_
 	return wrapped;
 }
 
+#if PHP_VERSION_ID >= 80400
+zend_string* xdebug_wrap_closure_location_around_function_name(zend_op_array *opa, zend_string *fname)
+{
+	zend_string *tmp, *tmp_loc_info;
+
+	if (ZSTR_VAL(fname)[ZSTR_LEN(fname) - 1] != '}') {
+		return zend_string_copy(fname);
+	}
+
+	tmp = zend_string_init(ZSTR_VAL(fname), strlen("{closure"), false);
+
+	tmp_loc_info = zend_strpprintf(
+		0,
+		"%s:%s:%d-%d}",
+		ZSTR_VAL(tmp),
+		ZSTR_VAL(opa->filename),
+		opa->line_start,
+		opa->line_end
+	);
+
+	zend_string_release(tmp);
+
+	return tmp_loc_info;
+}
+#else
 zend_string* xdebug_wrap_closure_location_around_function_name(zend_op_array *opa, zend_string *fname)
 {
 	zend_string *tmp, *tmp_loc_info;
@@ -781,6 +851,7 @@ zend_string* xdebug_wrap_closure_location_around_function_name(zend_op_array *op
 
 	return tmp_loc_info;
 }
+#endif
 
 static void xdebug_declared_var_dtor(void *dummy, void *elem)
 {
